const axios = require("axios")
const logger = require("darla")
const fs = require("fs")

logger.setLevel("console", "warn")

const ENGINE_NODE = "https://us.engine.rishipanthee.com/"
const DISCORD_WEBHOOK_URL = ""
const SYMBOL_TO_MONITOR = "SWAP.HIVE"

function getBlockEngine(blockNumber) {
  const query = { 'id': 0, jsonrpc: '2.0', method: 'getBlockInfo', params: { blockNumber: blockNumber } }
  let nextBlock = false
  axios.post(ENGINE_NODE + 'blockchain', query).then((res) => {
    if (res.data.result) {
      const block = res.data.result
      nextBlock = true
      logger.info(blockNumber)
      fs.writeFileSync("./state.json", JSON.stringify({ "lastParsedBlock": blockNumber }))
      parseBlockEngine(block)
    }
  }).finally(() => {
    if (nextBlock) {
      setTimeout(() => {
        getBlockEngine(blockNumber + 1)
      }, 0.5 * 1000)
    } else {
      setTimeout(() => {
        getBlockEngine(blockNumber)
      }, 1.5 * 1000)
    }
  }).catch((err) => {
    logger.error(`Unable to get latest Hive-Engine block. ${err}`)
  })
}


async function parseBlockEngine(block) {
  if (block.transactions.length !== 0) {
    for (const trx of block.transactions) {
      try {
        processTransaction(trx)
      } catch (err) {
        logger.error("Error noticed on a trx", err)
      }
    }
  }

}

async function processTransaction(trx) {
  if (trx.contract === "marketpools" && trx.action === "swapTokens") {
    const data = { quantityIn: "", symbolIn: "", userIn: "", quantityOut: "", symbolOut: "", userOut: "", feePaid: "", feeSymbol: "", trxID: "" }
    let logs = []
    try {
      logs = JSON.parse(trx.logs)
      if (!Array.isArray(logs.events)) {
        return
      }
    } catch (e) {
      logger.warn("Could not parse logs, most likely missing")
      logger.debug("The logs are : ", trx.logs)
      return
    }
    for (let event of logs.events) {
      if (event.event === "transferToContract") {
        data.quantityIn = event.data.quantity
        data.symbolIn = event.data.symbol
        data.userIn = event.data.from
      }

      if (event.event === "transferFromContract") {
        data.quantityOut = event.data.quantity
        data.symbolOut = event.data.symbol
        data.userOut = event.data.to
      }

      if (event.event === "swapTokens") {
        data.feePaid = event.data.fee.amount
        data.feeSymbol = event.data.fee.symbol
      }
    }

    if (data.symbolIn !== SYMBOL_TO_MONITOR || data.symbolOut !== SYMBOL_TO_MONITOR) {
      return
    }

    data.trxID = trx.transactionId

    const send = {
      "content": null,
      "embeds": [
        {
          "title": `${SYMBOL_TO_MONITOR} was traded by ${data.userIn} `,
          "url": `https://he.dtools.dev/tx/${data.trxID}`,
          "color": 16711680,
          "fields": [
            {
              "name": "Input Tokens",
              "value": data.quantityIn + " " + data.symbolIn
            },
            {
              "name": "Output Token",
              "value": data.quantityOut + " " + data.symbolOut
            },
            {
              "name": "Fees",
              "value": data.feePaid + " " + data.feeSymbol
            }
          ],
          "author": {
            "name": `Noticed activity on ${data.symbolIn}:${data.symbolOut} pool`,
            "icon_url": `https://images.hive.blog/u/${data.userIn}/avatar`
          },
          "footer": {
            "text": "Trade successfully completed.",
            "icon_url": "https://ipfspublic.rishipanthee.com/ipfs/QmWRqmJ1zmk5bYGoB6jntJzh3qt3mxcZhoByeZAgFjXD2W"
          }
        }
      ]
    }

    try {
      await axios.post(DISCORD_WEBHOOK_URL, send)
    } catch (err) {
      logger.error('Error posting to discord', err)
    }

  }
}


async function getStartStreamBlockEngine() {
  return new Promise((resolve, reject) => {
    const currentState = JSON.parse(fs.readFileSync("./state.json").toString())
    if (currentState.lastParsedBlock && currentState.lastParsedBlock != -1) {
      return resolve(currentState.lastParsedBlock)
    }
    const query = { 'jsonrpc': '2.0', 'method': 'getLatestBlockInfo', 'id': 0 }
    axios.post(ENGINE_NODE + 'blockchain', query).then((result) => {
      return resolve(result.data.result.blockNumber)
    }).catch((err) => {
      return reject(err)
    })
  })
}


async function startStreaming() {
  const startBlockHE = await getStartStreamBlockEngine()
  getBlockEngine(startBlockHE)
}


startStreaming()